package javax.rtgedition;

/***********************************************
 *  RTG, University of Pennsylvania
  * Author:	Dave Arney	11/07/01
  *                    Woongsup Kim
  * A general problem still to be fixed is that
  * millis and nanos are not kept in canonical
  * form. -DA 11/07/01
  **********************************************/

import java.util.Date;
import java.lang.String;

public class AbsoluteTime extends HighResolutionTime{

  Date date;

  /* Constructors */


  public AbsoluteTime() {
/* Equivalant to AbsoluteTime(0,0) */
	this(0,0);
  }

  public AbsoluteTime(AbsoluteTime time){
  	if(time == null){	
		millis = 0;
		nanos = 0;
	}
	else {
  		millis = time.getMilliseconds();
		nanos = time.getNanoseconds();
	}
  }

  public AbsoluteTime(java.util.Date date){
  	millis = date.getTime();	
	nanos = 0;
  }

  public AbsoluteTime(long millis, int nanos){
  	this.millis = millis;
	this.nanos = nanos;
  }

  /* Public Methods */
  public AbsoluteTime absolute(Clock clock, AbsoluteTime destination){
  	if(clock == null)
		clock = RTClock.getRealtimeClock();
	if(destination == null){
		return this;
	}
  	destination.set(this);
	return destination;
  }

  public AbsoluteTime add(long millis, int nanos){
  	long m = millis + getMilliseconds();
	int n = nanos + getNanoseconds();
	return new AbsoluteTime(m, n);
  }

  public AbsoluteTime add(long millis, int nanos, AbsoluteTime destination){
  	long m;
	int n;
	
	m = this.millis + millis;
	n = this.nanos + nanos;
	
  	if(destination == null){
		destination = new AbsoluteTime();
	}
	destination.set(m,n);
	return destination;
  }

  public final AbsoluteTime add(RelativeTime time){
	AbsoluteTime ab;
	long m;
	int n;

	if(time == null){
		return this;
	}
	ab = new AbsoluteTime();
	m = millis + time.getMilliseconds();
	n = nanos + time.getNanoseconds();
	ab.set(m,n);
	return ab;
  }

  public AbsoluteTime add(RelativeTime time, AbsoluteTime destination){
	AbsoluteTime ab;
	long m;
	int n;

	if(time == null){
		destination = this;
		return this;
	}
	if(destination == null){
		destination = new AbsoluteTime();
	}
	m = millis + time.getMilliseconds();
	n = nanos + time.getNanoseconds();
	destination.set(m,n);

	return destination;
  }

  public java.util.Date getDate(){
  	return new java.util.Date(getMilliseconds());
  }

  public void set(java.util.Date date){
  	millis = date.getTime();
	nanos = 0;
  }

  public final AbsoluteTime subtract(RelativeTime time){
	long m;
	int n;

	if(time == null){
		return this;
	}
	m = millis - time.getMilliseconds();
	n = nanos - time.getNanoseconds();

	AbsoluteTime r = new AbsoluteTime(m,n);
	return r;
  }

  public AbsoluteTime subtract(RelativeTime time, AbsoluteTime destination){
  	if(time == null){
		destination = new AbsoluteTime(millis, nanos);
		return destination;
	}
  	if(destination == null)
		destination = new AbsoluteTime(millis, nanos);
	long m = millis - time.getMilliseconds();
	int n = nanos - time.getNanoseconds();
	destination.set(m,n);
	return destination;
  }
  
  public final RelativeTime substract(AbsoluteTime time){
  	if(time == null){
		return new RelativeTime(0,0);
	}
  	long m = millis- time.getMilliseconds();
	int n = nanos - time.getNanoseconds();

	return new RelativeTime(m,n);
  }

  public RelativeTime subtract(AbsoluteTime time, RelativeTime destination){
  	long m;
	int n;
	
	if(time == null){
		destination = new RelativeTime(0,0);
		return destination;
	}
	if(destination == null){
		destination = new RelativeTime();
	}
	m = millis - time.getMilliseconds();
	n = nanos - time.getNanoseconds();

	destination.set(m,n);

	return destination;
  }


  public String toString(){
        Date date = getDate();
        StringBuffer  strm= new StringBuffer(date.toString());
        strm.append(":");
        strm.append(nanos);
        strm.append(" nanoseconds");
        return strm.toString();
  }

  
} 
