package javax.rtgedition;

/***********************************************
 *  RTG, University of Pennsylvania
  * Author:	Dave Arney	11/08/01
  *                    Woongsup Kim
  **********************************************/

import java.lang.String;

public class RelativeTime extends HighResolutionTime {

 protected int frequency;

  public RelativeTime() {
  	millis = 0;
	nanos = 0;
	frequency = 1;
  }

  public RelativeTime(long millis, int nanos) {
  	this.millis = millis;
	this.nanos = nanos;
	this.frequency = 1;
  }

  public RelativeTime(RelativeTime time) {
  	this.millis = time.getMilliseconds();
	this.nanos = time.getNanoseconds();
	this.frequency = 1;
  }

  public AbsoluteTime absolute(Clock clock, AbsoluteTime destination){
	AbsoluteTime ab;

	if(clock == null)
		clock = RTClock.getRealtimeClock();
	ab = clock.getTime();
	if(destination == null){
		destination = new AbsoluteTime();
	}
	destination.set(ab.getMilliseconds(), ab.getNanoseconds());
	return destination;
  }

  public RelativeTime add(long millis, int nanos){
  	long m;
	int n;

	RelativeTime r = new RelativeTime();
  	m = this.millis + millis;
	n = this.nanos + nanos;
	r.set(m,n);
	return r;
  }

  public RelativeTime add(long millis, int nanos, RelativeTime destination){
  	if(destination == null){
		destination = new RelativeTime();
		
	}
	destination.set(this.millis + millis, this.nanos + nanos);
	return destination;
  }

  public final RelativeTime add(RelativeTime time){
  	if(time == null){
		return this;
	}
  	RelativeTime r = new RelativeTime();

	r.set(this.millis + time.getMilliseconds(), this.nanos + time.getNanoseconds());
	return r;
  }

  public RelativeTime add(RelativeTime time, RelativeTime destination){
  	if(time == null){
		destination = this;
		return this;
	}
  	if(destination == null){
		destination = new RelativeTime();
	}
	destination.set(this.millis + time.getMilliseconds(), this.nanos + time.getNanoseconds());
	return destination;
  }

  public void addInterarrivalTo(AbsoluteTime destination){
  	destination.add(this, destination);
  }

  public RelativeTime getInterarrivalTime(RelativeTime destination){
  	return destination;
  }

  public final RelativeTime subtract(RelativeTime time){
  	RelativeTime ab = new RelativeTime();
	if(time == null){
		return this;
	}
	
	long mi = time.getMilliseconds();
	int na = time.getNanoseconds();
	if(millis > mi) 
		mi = millis - mi;
	else 
		mi = mi - millis;
	ab.set(mi, 0);
	return ab;
  }

  public RelativeTime subtract(RelativeTime time, RelativeTime destination){
	if(time == null){
		return this;
	}
  	if(destination == null){
		destination = new RelativeTime();	
	}
	long mi = time.getMilliseconds();
	int na = time.getNanoseconds();
	if(millis > mi) 
		mi = millis - mi;
	else 
		mi = mi - millis;
	destination.set(mi, 0);
	return destination;
  }

/*
  public String toString(){
  	return new String(millis);
  }
  */
}
