package javax.rtgedition;

/***********************************************
 *  RTG, University of Pennsylvania
  * Author:	Dave Arney	11/07/01
  *                    Woongsup Kim
  **********************************************/

public abstract class Timer extends AsyncEvent{
    protected RealtimeThread thread;
    protected Clock clock;
    protected boolean isEnabled;
    protected AbsoluteTime fireTime;

  protected Timer(HighResolutionTime c, AsyncEventHandler handler){
          disable();
        if (c == null)
        	clock = Clock.getRealTimeClock();
        else
        	clock = c;
        this.setHandler(handler);
        computeFireTime(t);
  }

  public ReleaseParameters createReleaseParameters() {
          RelativeTime cost = new RelativeTime(0,0);
        RelativeTime deadline=new RelativeTime(0,0);

        ReleaseParameters rp=new AperiodicParameters(cost,deadline,null,null);
        return rp;
    }

  public void disable() {
  	isEnabled = false;
    }

  public void enable() {
	synchronized(this) {
		isEnabled = true;
		notifyAll();
		if (thread != null) {
		thread.interrupt();
		}
	}
  }

  public Clock getClock() {
  	return clock;
  }

  public AbsoluteTime getFireTime() {
  	return fireTime;
  }

  public void reschedule(HighResolutionTime time){
  	computeFireTime(time);
  }


  public void start () {
	enable();
	thread = new RealtimeThread() {
		public void run() {
			for(;;) {
				try {
					RealtimeThread.sleep(Timer.this.clock, Timer.this.fireTime);
					synchronized(Timer.this) {
						if (isEnabled) {
							Timer.this.fire();
							Timer.this.thread = null;
							return;
						} else {
							Timer.this.wait();
						}
					}
				} catch(InterruptedException e) {
				}
			}
		}
	};
	thread.setDaemon(true);
	thread.start();
  }


  void computeFireTime(HighResolutionTime t) {
		fireTime = t.absolute(clock, fireTime);
  }

} 
