/*
 * Copyright (c) 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

/*=========================================================================
 * SYSTEM:    KVM
 * SUBSYSTEM: Global definitions
 * FILE:      global.c
 * OVERVIEW:  Global system-wide definitions.
 * AUTHOR:    Antero Taivalsaari, Sun Labs
 *            Many others since then...
 *=======================================================================*/

/*=========================================================================
 * COMMENT: 
 * This file contains declarations that do not belong to any 
 * particular structure or subsystem of the VM. There are additional 
 * global variable declarations in other files.
 *=======================================================================*/

/*=========================================================================
 * Local include files
 *=======================================================================*/

#include <global.h>

/*=========================================================================
 * Miscellaneous global variables
 *=======================================================================*/

/* Shared string buffer that is used internally by the VM */
/* NOTE: STRINGBUFFERSIZE is defined in main.h */
char str_buffer[STRINGBUFFERSIZE];

/*=========================================================================
 * Global execution modes
 *=======================================================================*/

/*  Flags for toggling certain global modes on and off */
int Verbose_On;    /* Legacy flag -- to be removed */
int Small_Heap;
bool_t JamEnabled;
bool_t JamRepeat;

/*========================================================================
 * Runtime flags for choosing different tracing/debugging options.
 * All these options make the system very verbose. Turn them all off
 * for normal VM operation.
 *=======================================================================*/

#if INCLUDEDEBUGCODE
                      
/*  Turns memory allocation tracing on/off */
int tracememoryallocation = 0;

/*  Turns garbage collection tracing on/off */
int tracegarbagecollection = 0;
int tracegarbagecollectionverbose = 0;

/*  Turns tracing of class loading on/off */
int traceclassloading = 0;
int traceclassloadingverbose = 0;

/*  Turns verifier tracing on/off */
int traceverifier = 0;

/*  Turns stack map tracing on/off */
int tracestackmaps = 0;

/*  Turns bytecode execution tracing on/off */
int tracebytecodes = 0;

/* Turns method call tracing on/off */
int tracemethodcalls = 0;
int tracemethodcallsverbose = 0;

/* Trace stack frames being pushed and popped */
int traceframes = 0;

/* Trace stack chunks being allocated and deallocated */
int tracestackchunks = 0;

/*  Turns exception handling tracing on/off */
int traceexceptions = 0;

/* Turns thread creation/switching/activation/suspension tracing on/off */
int tracethreading = 0; 

/* Turns monitor wait list addition/release tracing on/off */
int tracemonitors = 0;  

/*  Turns event handling tracing on/off */
int traceevents = 0;

/*  Turns networking tracing on/off */
int tracenetworking = 0;

#endif /* INCLUDEDEBUGCODE */

/*=========================================================================
 * Error handling definitions
 *=======================================================================*/

#if NEED_GLOBAL_JUMPBUFFER
jmp_buf *topJumpBuffer;
int     *topJumpBuffer_value;
#endif

