/*
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */
/*=========================================================================
 * SYSTEM:    KVM
 * SUBSYSTEM: Debugger
 * FILE:      debuggerCommands.h
 * OVERVIEW:  Table of commands that can be sent from the debugger
 *            to the KVM
 * AUTHOR:    Bill Pittore, Sun Labs
 *=======================================================================*/

typedef void (*CommandHandler)(PacketInputStream *, PacketOutputStream *);

void VirtualMachine_AllClasses(PacketInputStream *, PacketOutputStream *);
void VirtualMachine_AllThreads(PacketInputStream *, PacketOutputStream *);
void VirtualMachine_Suspend(PacketInputStream *, PacketOutputStream *);
void VirtualMachine_Resume(PacketInputStream *, PacketOutputStream *);
void VirtualMachine_Exit(PacketInputStream *, PacketOutputStream *);

void ReferenceType_GetValues(PacketInputStream *, PacketOutputStream *);

void ClassType_SuperClass(PacketInputStream *, PacketOutputStream *);
static void ClassType_SetValues(PacketInputStream *, PacketOutputStream *);

void Method_LineTable(PacketInputStream *, PacketOutputStream *);

void ObjectReference_ReferenceType(PacketInputStream *, PacketOutputStream *);

static void ObjectReference_GetValues(PacketInputStream *, PacketOutputStream *);
static void ObjectReference_SetValues(PacketInputStream *, PacketOutputStream *);

void StringReference_Value(PacketInputStream *, PacketOutputStream *);

void ThreadReference_Name(PacketInputStream *, PacketOutputStream *);
void ThreadReference_Suspend(PacketInputStream *, PacketOutputStream *);
void ThreadReference_Resume(PacketInputStream *, PacketOutputStream *);
void ThreadReference_Status(PacketInputStream *, PacketOutputStream *);
static void ThreadReference_Frames(PacketInputStream *, PacketOutputStream *);
void ThreadReference_FrameCount(PacketInputStream *in, PacketOutputStream *out);
void ThreadReference_SuspendCount(PacketInputStream *, PacketOutputStream *);

void ArrayReference_Length(PacketInputStream *, PacketOutputStream *);
void ArrayReference_GetValues(PacketInputStream *, PacketOutputStream *);
void ArrayReference_SetValues(PacketInputStream *, PacketOutputStream *);

static void EventRequest_Set(PacketInputStream *, PacketOutputStream *);
static void EventRequest_Clear(PacketInputStream *, PacketOutputStream *);
void EventRequest_ClearAllBreakpoints(PacketInputStream *, PacketOutputStream *);

void StackFrame_GetValues(PacketInputStream *, PacketOutputStream *);
static void StackFrame_SetValues(PacketInputStream *, PacketOutputStream *);

void KVMCommands_HandShake(PacketInputStream *, PacketOutputStream *);
void KVMCommands_GetSteppingInfo(PacketInputStream *, PacketOutputStream *);
void KVMCommands_Stepping_Event(PacketInputStream *, PacketOutputStream *);

void nop(PacketInputStream *, PacketOutputStream *);

void *VirtualMachine_Cmds[] = { (void *)16
    ,(void *)nop        /* VirtualMachine_SendVersion */
    ,(void *)nop        /* VirtualMachine_ClassesBySignature */
    ,(void *)VirtualMachine_AllClasses
    ,(void *)VirtualMachine_AllThreads
    ,(void *)nop        /* VirtualMachine_TopLevelThreadGroups */
    ,(void *)nop        /* dispose */
    ,(void *)nop        /* VirtualMachine_IDSizes */
    ,(void *)VirtualMachine_Suspend
    ,(void *)VirtualMachine_Resume
    ,(void *)VirtualMachine_Exit
    ,(void *)nop        /* VirtualMachine_CreateString */
    ,(void *)nop        /* VirtualMachine_Capabilities */
    ,(void *)nop        /* VirtualMachine_ClassPaths */
    ,(void *)nop        /* VirtualMachine_DisposeObjects */
    ,(void *)nop        /* holdEvents */
    ,(void *)nop        /* releaseEvents */
};

void *ReferenceType_Cmds[] = { (void *)11
    ,(void *)nop        /* ReferenceType_Signature */
    ,(void *)nop        /* ReferenceType_ClassLoader */
    ,(void *)nop        /* ReferenceType_Modifiers */
    ,(void *)nop        /* ReferenceType_Fields */
    ,(void *)nop        /* ReferenceType_Methods */
    ,(void *)ReferenceType_GetValues
    ,(void *)nop        /* ReferenceType_SourceFile */
    ,(void *)nop        /* nestedTypes */
    ,(void *)nop        /* ReferenceType_Status */
    ,(void *)nop        /* ReferenceType_Interfaces */
    ,(void *)nop        /* classObject */
};

void *ClassType_Cmds[] = { (void *)0x4
    ,(void *)ClassType_SuperClass
    ,(void *)ClassType_SetValues
    ,(void *)nop        /* invokeStatic */
    ,(void *)nop        /* invokeStatic */
};

void *ArrayType_Cmds[] = { (void *)0x1
                          ,(void *)nop /* newInstance */
};

void *Field_Cmds[] = { (void *)0x0 };

void *Method_Cmds[] = { (void *)0x3
    ,(void *)nop /* Method_LineTable */
    ,(void *)nop /* variable Table */
    ,(void *)nop /* bytecodes */
};

void *ObjectReference_Cmds[] = { (void *)0x9
    ,(void *)ObjectReference_ReferenceType
    ,(void *)ObjectReference_GetValues
    ,(void *)ObjectReference_SetValues
    ,(void *)NULL /* no longer used */
    ,(void *)nop  /* monitorInfo */
    ,(void *)nop  /* invokeInstance */
    ,(void *)nop  /* disableCollection */
    ,(void *)nop  /* enableCollection */
    ,(void *)nop  /* isCollected */
    };

void *StringReference_Cmds[] = { (void *)0x1
    ,(void *)StringReference_Value };

void *ThreadReference_Cmds[] = { (void *)12
    ,(void *)ThreadReference_Name
    ,(void *)ThreadReference_Suspend
    ,(void *)ThreadReference_Resume
    ,(void *)ThreadReference_Status
    ,(void *)nop  /* ThreadReference_ThreadGroup */
    ,(void *)ThreadReference_Frames
    ,(void *)ThreadReference_FrameCount
    ,(void *)nop  /* ownedMonitors */
    ,(void *)nop  /* currentContendedMonitor */
    ,(void *)nop  /* stop */
    ,(void *)nop  /* interrupt */
    ,(void *)ThreadReference_SuspendCount
    };

void *ThreadGroupReference_Cmds[] = { (void *)3
    ,(void *)nop  /* ThreadGroupReference_Name */
    ,(void *)nop  /* ThreadGroupReference_Parent */
    ,(void *)nop  /* ThreadGroupReference_Children */
    };

void *ClassLoaderReference_Cmds[] = { (void *)0x1
    ,(void *)nop  /* visibleClasses */
};

void *ArrayReference_Cmds[] = { (void *)0x3
    ,(void *)ArrayReference_Length
    ,(void *)ArrayReference_GetValues
    ,(void *)ArrayReference_SetValues};

void *EventRequest_Cmds[] = { (void *)0x3
    ,(void *)EventRequest_Set
    ,(void *)EventRequest_Clear
    ,(void *)EventRequest_ClearAllBreakpoints};

void *StackFrame_Cmds[] = { (void *)0x3
    ,(void *)StackFrame_GetValues
    ,(void *)StackFrame_SetValues
    ,(void *)nop  /* StackFrame_ThisObject */
};

void *ClassObjectReference_Cmds[] = { (void *)1
    ,(void *)nop  /* reflectedType */
};

void *KVMCommands[] = { (void *)3
    ,(void *)KVMCommands_HandShake
    ,(void *)KVMCommands_GetSteppingInfo
    ,(void *)nop  /* command from KVM to debug agent - KVMCommands_Stepping_Event */
};

