/*
 * Copyright (c) 1998-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

/*=========================================================================
 * KVM
 *=========================================================================
 * SYSTEM:    KVM
 * SUBSYSTEM: Main program
 * FILE:      main.c
 * OVERVIEW:  Main program for command-line based environments
 * AUTHOR:    Antero Taivalsaari, Sun Labs
 *            Edited by Doug Simon 11/1998
 *            JAM integration by Sheng Liang
 *
 * NOTE:      KVM does not have a portable main() function.  This is
 *            because the VM may be used in very different kinds of 
 *            target environments.  Some of the environments may provide
 *            command line support, while many don't; some environments
 *            may launch the VM from a GUI or a micro-browser, etc.
 *
 *            The portable VM startup and shutdown operations are defined
 *            in file VmCommon/src/StartJVM.c. The main() function defined
 *            in this file is applicable only to those target systems that
 *            support VM startup from a command line.
 *=======================================================================*/

/*=========================================================================
 * Include files
 *=======================================================================*/

#include <global.h>

#if USE_JAM
#include <jam.h>
#endif

/*=========================================================================
 * Functions
 *=======================================================================*/

void printHelpText() {
#if !USE_JAM

    fprintf(stdout, "Usage: kvm <-options> <classfile>\n");
    fprintf(stdout, "Options:\n");
    fprintf(stdout, "  -version\n");
    fprintf(stdout, "  -classpath <filepath>\n");

#if ENABLE_JAVA_DEBUGGER
    fprintf(stdout, "  -debugger\n");
    fprintf(stdout, "  -port <port number>\n");
#endif /* ENABLE_JAVA_DEBUGGER */

#if INCLUDEDEBUGCODE
    fprintf(stdout, "  -traceallocation\n");
    fprintf(stdout, "  -tracegc\n");
    fprintf(stdout, "  -tracegcverbose\n");
    fprintf(stdout, "  -traceclassloading\n");
    fprintf(stdout, "  -traceclassloadingverbose\n");
    fprintf(stdout, "  -traceverifier\n");
    fprintf(stdout, "  -tracestackmaps\n");
    fprintf(stdout, "  -tracebytecodes\n");
    fprintf(stdout, "  -tracemethods\n");
    fprintf(stdout, "  -tracemethodsverbose\n");
    fprintf(stdout, "  -tracestackchunks\n");
    fprintf(stdout, "  -traceframes\n");
    fprintf(stdout, "  -traceexceptions\n");
    fprintf(stdout, "  -traceevents\n");
    fprintf(stdout, "  -tracemonitors\n");
    fprintf(stdout, "  -tracethreading\n");
    fprintf(stdout, "  -tracenetworking\n");
    fprintf(stdout, "  -traceall (activates all tracing options above)\n");
#endif /* INCLUDEDEBUGCODE */

#endif /* !USE_JAM */
}

#ifndef BUILD_VERSION
#define BUILD_VERSION "generic"
#endif

int main (int argc, char* argv[]) {
    int result;

#if USE_JAM
    char *jamInstalledAppsDir = "./instapps";
#endif

    Verbose_On = FALSE;
    JamEnabled = FALSE;
    JamRepeat = FALSE;
        
    while (argc > 1) {    
        if (strcmp(argv[1], "-version") == 0) {
            fprintf(stdout, "Version: %s\n", BUILD_VERSION);
            exit(1);           
        } else if (strcmp(argv[1], "-help") == 0) {
            printHelpText();
            exit(0);
        } else if (strcmp(argv[1], "-verbose") == 0) {  /* To be removed */
            Verbose_On = TRUE;
            argv++; argc--;

#if ENABLE_JAVA_DEBUGGER
        } else if (strcmp(argv[1], "-debugger") == 0) {
            debuggerActive = TRUE;
            argv++; argc--;
        } else if (strcmp(argv[1], "-port") == 0) {
            debuggerPort = (short)atoi(argv[2]);
            argv+=2; argc -=2;
#endif

        } else if ((strcmp(argv[1], "-classpath") == 0) && argc > 2) {
            if (JamEnabled) {
                fprintf(stderr, KVM_MSG_CANT_COMBINE_CLASSPATH_OPTION_WITH_JAM_OPTION);
                exit(1);
            }
            UserClassPath = argv[2];
            argv+=2; argc -=2;

#if INCLUDEDEBUGCODE

        } else if (strcmp(argv[1], "-traceallocation") == 0) {
            tracememoryallocation = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracegc") == 0) {
            tracegarbagecollection = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracegcverbose") == 0) {
            tracegarbagecollectionverbose = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceclassloading") == 0) {
            traceclassloading = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceclassloadingverbose") == 0) {
            traceclassloadingverbose = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceverifier") == 0) {
            traceverifier = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracestackmaps") == 0) {
            tracestackmaps = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracebytecodes") == 0) {
            tracebytecodes = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracemethods") == 0) {
            tracemethodcalls = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracemethodsverbose") == 0) {
            tracemethodcallsverbose = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracestackchunks") == 0) {
            tracestackchunks = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceframes") == 0) {
            traceframes = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceexceptions") == 0) {
            traceexceptions = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceevents") == 0) {
            traceevents = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracemonitors") == 0) {
            tracemonitors = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracethreading") == 0) {
            tracethreading = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-tracenetworking") == 0) {
            tracenetworking = 1;
            argv++; argc--;
        } else if (strcmp(argv[1], "-traceall") == 0) {
            tracememoryallocation = 1;
            tracegarbagecollection = 1;
            tracegarbagecollectionverbose = 1;
            traceclassloading = 1;
            traceclassloadingverbose = 1;
            traceverifier = 1;
            tracestackmaps = 1;
            tracebytecodes = 1;
            tracemethodcalls = 1;
            tracemethodcallsverbose = 1;
            tracestackchunks = 1;
            traceframes = 1;
            traceexceptions = 1;
            traceevents = 1;
            tracemonitors = 1;
            tracethreading = 1;
            tracenetworking = 1;
            argv++; argc--;

#endif /* INCLUDEDEBUGCODE */

#if USE_JAM
        } else if (strcmp(argv[1], "-jam") == 0) {
            JamEnabled = TRUE;
            argv++; argc--;
        } else if (JamEnabled && (strcmp(argv[1], "-repeat") == 0)) {

#if ENABLE_JAVA_DEBUGGER
            if (debuggerActive) {
                fprintf(stderr,
                    KVM_MSG_CANT_COMBINE_DEBUGGER_OPTION_WITH_REPEAT_OPTION);
                exit(1);
            }
#endif
            JamRepeat = TRUE;
            argv++; argc--;
        } else if (JamEnabled && (strcmp(argv[1], "-appsdir") == 0)
                  && argc > 2) {
            jamInstalledAppsDir = argv[2];
            argv+=2; argc-=2;
#endif /* USE_JAM */

        } else {
            break;
        }
    }

    /* Skip program name */
    argc--;
    argv++;

    if (UserClassPath == NULL && !JamEnabled) { 
        UserClassPath = getenv("classpath");
        if (UserClassPath == NULL) { 
            /*  Just in case environment variable reading is case sensitive */
            UserClassPath = getenv("CLASSPATH");
            if (UserClassPath == NULL) { 
                fprintf(stdout, KVM_MSG_CLASSPATH_NOT_SET);
                UserClassPath = ".";
            }
        }
    }

#if USE_JAM
    if (JamEnabled) {
        if (argc != 1 || 
            ((strncmp(argv[0], "http://", 7) != 0) &&
             (strncmp(argv[0], "file://", 7) != 0))) {
            fprintf(stderr, 
                    KVM_MSG_EXPECTING_HTTP_OR_FILE_WITH_JAM_OPTION);
            exit(1);
        }
        JamInitialize(jamInstalledAppsDir);
        do {
            result = JamRunURL(argv[0], JamRepeat);
            if (result == JAM_RETURN_ERR) {
                break;
            }
        } while(JamRepeat);
    } else
#endif /* USE_JAM */

    {
        /* Call the portable KVM startup routine */
        result = StartJVM(argc, argv);

        /* If no classfile was provided, print help text */
        if (result == -1) printHelpText();
    }

    return result;
}

